#
# ident	"@(#)check_host.tcl	1.64	00/09/06 SMI"
#
# Copyright (c) 1996-2000 by Sun Microsystems, Inc.
# All rights reserved.
#
# check_host is a SSP command that determines whether the host, specified
# by the HOSTNAME env variable, is UP or DOWN.
# The host is considered UP, if all configured procs are running the O/S
# and the signature_block state is RUN. Otherwise, the host is considered
# DOWN.
# The signature block is read from the MIB, kept by the snmpd daemon.
#
# Usage: (Note: not all cmd line args are advertised)
#
#	check_host [-q] [-v] [-b] [-B] [-P] [-p proc] [-h hostname]
#	-q : quiet mode (Don't output "Host is . . ." message or
#		other messages to stdout or stderr. This is intended
#		for use from scripts and other programs,
#		which just check the check_host return status).
#	-v : verbose mode (log messages to syslog)
#	-b : reserved for bringup (to determine whether to do a hpost -C)
#	-B : reserved for bringup. Log sigs in the domain messages file.
#       -P : reserved for ignoring unknown signature/state; e.g., called 
#		from power failure recovery when the system boards are off
#	-p proc : check only signatures on processor proc
#	-h hostname : treat hostname as SUNW_HOSTNAME
#
# NOTE -- This command has been internationalized.  There have been
#	several changes made to output strings.  These changes have
#	been made to facilitate internationalization.  
#

#
# check_host return values:
#
set HOST_IS_UP 0
set HOST_IS_DOWN 1
set CHECK_HOST_FAILED -1 ;# Problem (usually environment)--don't know if up/down


set CB_CONFIG_FILE $env(SSPVAR)/.ssp_private/cb_config


proc check_host args {
    global HOST_IS_UP
    global HOST_IS_DOWN
    global CHECK_HOST_FAILED
    global env
    set bHostDown 0

    # this is implictly handled in libssptcl, it should be set to
    # SUNW_SSP automatically, as a result.
    # textdomain %TEXT_DOMAIN%

    #
    # Good OS states
    #
    set OS(run) 1
    set OS(detached) 1
    set OS(callback) 1
    set OS(offline) 1
    set OS(quiesced) 1
    set OS(quiesce-in-progress) 1
    set OS(resume-in-progress) 1

    #
    # Good OBP states
    #
    set OBP(callback) 1

    #
    # Check command line arguments
    #
    set validArgs "-qbBvPp:h:"


    if { (([string length $args] == 1) && ($args == "-")) \
	    || [ catch {CLA_Parse Args $args $validArgs} errMsg ] } {
	# TRANSLATION_NOTE - %s is the error message (translated!)
	set fmt [gettext "Error: %s"]
	puts stderr [format "$fmt" "$errMsg"]
	puts stderr [gettext {usage: check_host [-q]}]
	if { [info exists Args(v)] || [info exists Args(b)] } {
	    logMessage "invalid command line arguments: $errMsg"
	}
	return $CHECK_HOST_FAILED
    }

    if { [info exists Args(q)] } {
	# Don't output any "Host is . . ." message or other messages
	# to stdout or stderr.
	set bQuiet 1
    } else {
	set bQuiet 0
    }

    if { [info exists Args(B)] } {
	# verbosity level for bringup
	set bVerbose 1
    } else {
	set bVerbose 0
    }

    if { [info exists Args(b)] } {
	# check_host was called by one of the following: ssp_startup, bringup,
	# actionsysclock, actionreboot, actiontemp, actionpower.
	# 
	# This is used to determine whether to configure the centerplane (-C)
	set bStartup 1
    } else {
	set bStartup 0
    }

    if { [info exists Args(p)] } {
	# check only signatures on these processors
	set bCheckOnlySigs 1
    } else {
	set bCheckOnlySigs 0
    }

    if { [info exists Args(v)] } {
	# verbose mode
	set bInternal 1
    } else {
	set bInternal 0
    }

    if { [info exists Args(P)] } {
	# check_host was called by PowerFailRebootact - unknown
	# signature is acceptable (Bug 4368475).
        set bIgnore_signature 1
    } else {
        set bIgnore_signature 0
    }


    #
    # Check SSP environment variables:
    #
    set ssp_env "SUNW_HOSTNAME SSPETC SSPVAR"

    foreach var $ssp_env {
	if { ![info exists env($var)] } {
	    if { $bQuiet == 0 } { 
		# TRANSLATION_NOTE - %s is environment variable name
		set fmt [gettext "Must set environment variable: '%s'"]
		puts stderr [format "check_host: $fmt" "$var"]
	    }
	    if { ($bVerbose == 1) || ($bStartup == 1) } {
		logMessage "Environment variable '$var' must be set."
	    }
	    return $CHECK_HOST_FAILED
	}
    }

    if { [info exists Args(h)] } {
	set env(SUNW_HOSTNAME) $Args(h)
    }

    # If -p is NOT present, use ping to quickly decide if the host is UP
    # (bug 4102350).
    # If ping failed, it doesn't necessary mean the host is down--it could 
    # be network traffic, so continue with more checking.
    # Arg p needs further checking to decide if the host is UP or DOWN.
    if { $bCheckOnlySigs == 0 } { 
	if { [catch { exec ping $env(SUNW_HOSTNAME) 3}  errmsg ] == 0  } {
	    if { $bQuiet == 0 } { 
		puts [gettext "Host is UP"]
	    }
	    return $HOST_IS_UP
	}
    }

    #
    # Get a SNMP session handle to talk to the snmpd
    #
    if { [catch {snmp session -community check_host } handle] != 0 } {
	# TRANSLATION_NOTE - %s is error result from snmp session
	if { $bQuiet == 0 } { 
		set fmt [gettext "Error creating snmp session - %s"]
		puts stderr [format "check_host: $fmt" "$handle"]
	}
	if { ($bVerbose == 1) || ($bStartup == 1) } {
	    logMessage "error creating snmp session - $handle"
	}
	return $CHECK_HOST_FAILED
    }

    #if { $bInternal == 1 } { 
    #	puts "  snmp session is $handle"
    #}

    #
    # Ping the Control Board to verify it's ready
    # If not ready, retry and hope it'll become available.
    # (bug 4102350)
    #
    set primary_cb [get_primary_cb_name [get_platform_name $env(SUNW_HOSTNAME)]]
    set CB_PING_COUNT 90 ;# 1 ping/second
    # (cb_reboot's "reboot_cycle_time" is 90 seconds)
	
    if { [catch { exec ping $primary_cb $CB_PING_COUNT}  errmsg ] != 0  } {
	# Can't reach control board
	# This could happen if SUNW_HOSTNAME is not set to a domain
	if { $bQuiet == 0 } { 
		puts stderr [gettext \
			"Cannot reach control board $primary_cb; perhaps SUNW_HOSTNAME is not set to a valid domain."]
	}
	if { ($bVerbose == 1) || ($bStartup == 1) } {
	    logMessage "Cannot reach control board $primary_cb; perhaps SUNW_HOSTNAME is not set to a valid domain."
	}
	return $CHECK_HOST_FAILED
    }


    #
    # Do we want the signature of just one cpu? (-p)
    #
    if { $bCheckOnlySigs == 1 } {
	return [ check_host_p $handle $Args(p) ]
    }

    #
    # Query snmpd for domain's sysboard and proc config list
    #
    set proc_list ""
    set brd_list ""
    set validhost 0

    if { [catch { $handle walk x "domainIndex domainName" {
	set indx [lindex [lindex $x 0] 2]
	set host [lindex [lindex $x 1] 2]
	if { $host == $env(SUNW_HOSTNAME) } {
	    # Got our domain name
	    set brd_list [lindex [lindex \
			[$handle get domainSysBrdConfig.$indx] 0] 2]
	    set proc_list [lindex [lindex \
			[$handle get domainProcConfig.$indx] 0] 2]
	    set validhost 1
	    break
	}
   }  } error ] != 0 }  {  
       if { $bQuiet == 0 } {
	   puts stderr $error
       }
       if { ($bVerbose == 1) || ($bStartup == 1) } {
	   logMessage "SNMP Cannot find domain $env(SUNW_HOSTNAME): $error"
       }
       $handle destroy
       return $CHECK_HOST_FAILED 
   } 

   #
   # Confirm if SUNW_HOSTNAME is set to a valid domain name
   #
   if { $validhost == 0 } {
       if { $bQuiet == 0 } {
	   # TRANSLATION_NOTE - %s is $SUNW_HOSTNAME
	   set fmt [gettext "%s is not valid domain"]
	   puts [format "$fmt" "$env(SUNW_HOSTNAME)"]
       }
       if { ($bVerbose == 1) || ($bStartup == 1) } {
	   logMessage "$env(SUNW_HOSTNAME) is not a valid domain"
       }
       $handle destroy
       return $HOST_IS_DOWN
   }

   #
   # Check that no sysboards have arb-stopped
   #
   set brd_list [ split $brd_list - ]

   if { $brd_list == "" } {
       if { $bQuiet == 0 } { 
	   puts [gettext "Host is DOWN"]
       }
       if { ($bVerbose == 1) } {
	   logDomainMessage "No SNMP sysboard config for this domain" "check_host"
       }
       $handle destroy
       return $HOST_IS_DOWN
   }

   if { $bInternal == 1 } { puts "  System Board list is $brd_list" }

   if { [catch { check_laarb $brd_list}  arb ] == 0  } {
       set stop_chk [ lindex $arb 1 ] 
       if { $stop_chk != 0 } {
	   if { $bQuiet == 0 } { 
	       puts [gettext "Host is DOWN"]
	   }
	   if { ($bVerbose == 1) || ($bStartup == 1) } {
	       set message [format "WARNING : arb-stop detected on system board %s" \
		       [lindex $arb 0]]
	       logDomainMessage $message "check_host"
	   }
	   $handle destroy
	   return $HOST_IS_DOWN
       }
   }

   #
   # Continue on with proc's signature and state
   #
   set proc_config_list [ split $proc_list - ]
   if { $proc_config_list == "" } {
       if { $bQuiet == 0 } { puts "Host is DOWN" }
       if { ($bVerbose == 1) || ($bStartup == 1) } {
	   logDomainMessage "No SNMP proc config for this domain" "check_host"
       }
       $handle destroy
       return $HOST_IS_DOWN
   }

   if { $bInternal == 1 } {
       puts "  Processor config list is $proc_config_list"
   }

   #
   # Build a good proc list from processors in the proc config list
   # marked good
   #
   set good_proc_list ""
   foreach i $proc_config_list {
       set b_num 0
       set p_num 0
       set p_status "F"
       scan [ split $i . ] "%d %d %s" b_num p_num p_status

       if { $p_status == "G" } {
	   set p [expr $b_num * 4 + $p_num]
	   lappend good_proc_list $p
       }
   }

   if { $good_proc_list == "" } {
       if { $bQuiet == 0 } { puts "Host is DOWN" }
       if { ($bVerbose == 1) || ($bStartup == 1) } {
	   logDomainMessage "No SNMP good proc config for this domain" \
		"check_host"
       }
       $handle destroy
       return $HOST_IS_DOWN
   }

   if { $bInternal == 1 } {
       puts "  Good processor config list is $good_proc_list"
   }

   #
   # Convert good proc list to good index list
   #
   set good_index_list ""
   set PROC_STATE_INDEXES "procStateGenNum procStateGenIndex"
   if { [ catch {$handle walk x  "$PROC_STATE_INDEXES"   {
       set num [lindex [lindex $x 0] 2]
       if { [ lsearch $good_proc_list $num ] >= 0 } {
	   # This proc belongs to this domain
	   set index [lindex [lindex $x 1] 2]
	   lappend good_index_list $index
       }
   }   } error ] == 0 }  {
       if { $good_index_list == "" } {
	   if { $bQuiet == 0 } { puts [gettext "Host is DOWN"] }
	   if { ($bVerbose == 1) || ($bStartup == 1) } {
	       logDomainMessage "No SNMP good proc config index for this domain" \
			"check_host"
	   }
	   $handle destroy
	   return $HOST_IS_DOWN
       }
   }
        
   if { $bInternal == 1 } { puts "  Good indexes are: $good_index_list" }

   if { $bStartup == 1 } {
       # with empty good_index_list because reading MIB failure, 
       # check_host_b returns UP
       return [ check_host_b $handle $good_index_list ]
   }

   #
   # Read the signature block (state and sig fields) for each proc
   # in good_index_list.
   #
   foreach index $good_index_list {
       if { [catch {$handle get "procStateGenPgmSignature.$index \
			procStateGenPgmState.$index" } data] != 0 } {
	   if { $bQuiet == 0 } { 
		# TRANSLATION_NOTE - %d is processor index
		set fmt [gettext \
			"Warning: cannot determine host signature, index %d"]
		puts stderr [format "check_host: $fmt" "$index"]
		set fmt [gettext "Error: %s"]
		puts stderr [format "$fmt" "$data"]
	   }
	   if { ($bVerbose == 1) || ($bStartup == 1) } {
	       logMessage "Warning: cannot determine host signature, index $i: $data"
	   }
	   # safer to assume Host is UP
	   break
       }
       set sig   [lindex [lindex $data 0] 2]
       set state [lindex [lindex $data 1] 2]

       if { $bInternal == 1 } {
	   puts "  Processor index $index signature: $sig state: $state"
       }

       if { ($sig == "os") && ([info exists OS($state)]) } {
	   continue
       } elseif { ($sig == "obp") && ([info exists OBP($state)]) } {
	   continue
       } else {
	   # Host is down
	   incr bHostDown
	   break
       }
   }

   if { $bHostDown } {
       if { $bQuiet == 0 } { puts "Host is DOWN" }
       dump_signatures $handle $good_index_list $bVerbose
       $handle destroy
       return $HOST_IS_DOWN
   } else {
       if { $bQuiet == 0 } { puts "Host is UP" }
       $handle destroy
       return $HOST_IS_UP
   }
}


#
# check_host_b:
#
# This procedure is used by bringup to decide whether to use the -C argument
# to configure the centerplane. Other components that call this to determine
# whether a host is up or down are:
# ssp_startup, actionsysclock, actionreboot, actiontemp, actionpower.
#
# This procedure checks state, signature, and heartbeat of each processor
# before determining if the Host is UP or DOWN.
# With an empty index_list, check_host_b returns UP
#
# Note: For check_host -b, it's safer to assume the host is UP.  That's
# because if bringup thinks all domains are down (determined by
# calling check_host -b for all domains), bringup does a centerplane
# reconfig (hpost -C), which hoses any domains that may be up.
#
proc check_host_b { SnmpSession index_list } {
    global HOST_IS_UP
    global HOST_IS_DOWN
    global CHECK_HOST_FAILED
    upvar bQuiet Quiet
    upvar bVerbose Verbose
    upvar bStartup Startup
    upvar bInternal Internal
    upvar bIgnore_signature Ignore_signature
    upvar Args Arrgg
    set sig ""

    foreach i $index_list {
	if { [catch {$SnmpSession get "procStateGenNum.$i \
		procStateGenPgmSignature.$i \
		procStateGenPgmState.$i procStateGenPgmSubState.$i"} data] != 0 } {
	    # Host is down ? it is safer to assume UP
	    # TRANSLATION_NOTE - %d is processor index
	    if { $bQuiet == 0 } { 
	   	 set fmt [gettext \
		"Warning: cannot determine host state and signature, Index %d"]
	    	puts [format "$fmt" "$i"]
	    	set fmt [gettext "Error: %s"]
	    	puts stderr [format "$fmt" "$data"]
	    }

	    if { ($Verbose == 1) || ($Startup == 1) } {
		logMessage "Warning: cannot determine host state and signature, index $i: $data"
	    }
	    dump_signatures $SnmpSession $index_list $Verbose
	    $SnmpSession destroy
	    return $HOST_IS_UP
	}
	set proc [lindex [lindex $data 0] 2]
	set sig [lindex [lindex $data 1] 2]
	set state [lindex [lindex $data 2] 2]
	set subState [lindex [lindex $data 3] 2]

	if { $Internal == 1 } {
	    puts "processor $proc, signature $sig, state $state, substate $subState"
	}


	#
	# Look at the processor signature state value and check accordingly.
	# Expected states:
	# post (1), download-helper (2), obp (3), os (4), or unknown (5)
	#
	switch $sig {
	    post {
		# Don't care. Note: bringup checks for hpost process
		if { $Quiet == 0 } {
		    # TRANSLATION_NOTE - $1 is CPU sig $2 is CPU state
		    puts [format [gettext {Host is DOWN, %1$ %2$s}] \
				"$sig" "$state"]
		}
		dump_signatures $SnmpSession $index_list $Verbose
		$SnmpSession destroy
		return $HOST_IS_DOWN
	    }

	    download-helper {
		if { $state == "run" } {
		    # Note: need validate just one proc
		    if { $Quiet == 0 } {
			# TRANSLATION_NOTE - %s is CPU sig (e.g. obp)
			set fmt [gettext "Host is ALIVE, %s running"]
			puts [format "$fmt" "$sig"]
		    }
		    $SnmpSession destroy
		    return $HOST_IS_UP
		} else {
		    # State is either unknown or exit
		    if { $Quiet == 0 } {
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			set fmt [gettext {Host is DOWN, %1$s %2$s}]
			puts [format "$fmt" "$sig" "$state"]
		    }
		    dump_signatures $SnmpSession $index_list $Verbose
		    $SnmpSession destroy
		    return $HOST_IS_DOWN
		}
	    }

	    obp {
		# Check state and subState. Note: need to validate just one proc
		if { ($state == "run") || ($state == "callback") } {
		    if { $Quiet == 0 } {
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			set fmt [gettext {Host is ALIVE, %1$s %2$s}]
			puts [format "$fmt" "$sig" "$state"]
		    }
		    $SnmpSession destroy
		    return $HOST_IS_UP
		}

		# State not running
		if { (($state == "exit") && \
			(($subState == "exit-reboot") || \
			($subState == "exit-panic-reboot"))) } {
		    # Reboot in progress
		    if { $Quiet == 0 } {
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			# %3$s is CPU subState
			set fmt [gettext {%1$s %2$s %3$s. Reboot in progress}]
			puts [format "$fmt" "$sig" "$state" "$subState"]
		    }
		    $SnmpSession destroy
		    return $HOST_IS_UP
		} elseif { $state == "booting" } {
		    # Boot in progress
		    if { $Quiet == 0 } { 
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			# %3$s is CPU subState
			set fmt [gettext {%1$s %2$s %3$s. OBP booting}]
			puts [format "$fmt" "$sig" "$state" "$subState"]
		    }
		    $SnmpSession destroy
		    return $HOST_IS_UP
		} elseif { (($state == "exit") && \
			($subState == "exit-halt") || \
			($subState == "exit-obp-reset")) || \
			($state == "watchdog") || \
			($state == "watchdog-sync") || \
			($state == "redmode") || \
			($state == "redmode-sync") } {
		    if { $Quiet == 0 } { 
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			# %3$s is CPU subState
			set fmt [gettext {Host is ACTIVE, %1$s %2$s %3$s}]
			puts [format "$fmt" "$sig" "$state" "$subState"]
		    }
		    $SnmpSession destroy
		    return $HOST_IS_UP
		} else {
		    # Bad OBP state
		    if { $Quiet == 0 } { 
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			# %3$s is CPU subState
			set fmt [gettext {Host is DOWN, %1$s %2$s %3$s}]
			puts [format "$fmt" "$sig" "$state" "$subState"]
		    }
		    dump_signatures $SnmpSession $index_list $Verbose
		    $SnmpSession destroy
		    return $HOST_IS_DOWN
		}
	    }

	    os {
		if { $state == "run" } {
		    # domain is active, continue
		} elseif { ($state == "watchdog-sync") || \
			($state == "xir-sync") || \
			($state == "detached") || \
			($state == "callback") || \
			($state == "offline") || \
			($state == "quiesced") || \
			($state == "quiesce-in-progress") || \
			($state == "resume-in-progress") } { \
		    # domain is active, continue
		} elseif { (($state == "exit") && \
			(($subState == "exit-panic1") || \
			($subState == "exit-panic2") || \
			($subState == "exit-reboot") || \
			($subState == "exit-panic-reboot"))) } {
		    # OS panic'ed and edd will reboot
		    if { $Quiet == 0 } {
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			# %3$s is CPU subState
			set fmt [gettext \
				{Host is ACTIVE, %$1s %2$s %3$s, Reboot in progress}]
			puts [format "$fmt" "$sig" "$state" "$subState"]
		    }
		    $SnmpSession destroy
		    return $HOST_IS_UP
		} else {
		    # OS hang! Host is Down
		    if { $Quiet == 0 } { 
			# TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
			# %3$s is CPU subState
			set fmt [gettext {Host is DOWN, %1$s %2$s %3$s}]
			puts [format "$fmt" "$sig" "$state" "$subState"]
		    }
		    # Unexpected state - log a message if state should not be
		    # unknown (Bug 4368475).
		    if { $Ignore_signature == 0 } {
			set fmt [gettext
			"Unexpected signature:%s, state:%s, subState:%s"]
			logDomainMessage [format
			"$fmt" "$sig" "$state" "$subState"] "check_host"
		    }
		    dump_signatures $SnmpSession $index_list $Verbose
		    $SnmpSession destroy
		    return $HOST_IS_DOWN
		}
	    }

	    default {
		# unknown sig
		if { $Internal == 1 } {
		    puts stderr "Error: unknown signature `$sig` on proc $proc"
		}
		if { $Quiet == 0 } {
		    # TRANSLATION_NOTE - %1$s is CPU sig, %2$s is CPU state
		    # %3$s is CPU subState
		    set fmt [gettext {Host is DOWN, %1$s %2$s %3$s}]
		    puts [format "$fmt" "$sig" "$state" "$subState"]
		}
		# Always dump signature if signature should not be unknown.
		# Bug 4368475
                if { $Ignore_signature == 1 } {
			dump_signatures $SnmpSession $index_list $Verbose
		} else {
			# Unexpected unknown signature - dump it
			dump_signatures $SnmpSession $index_list 1
		}
		$SnmpSession destroy
		return $HOST_IS_DOWN
            }
	}
    }

    # Host is either UP or coming UP
    if { $Quiet == 0 } {
	if { $sig == "os" } {
	    puts [gettext "Host is UP"]
	} else {
	    # TRANSLATION_NOTE - $s is CPU signature (e.g. obp)
	    set fmt [gettext "Host is ALIVE, %s running"]
	    puts [format "$fmt" "$sig"]
	}
    }
    $SnmpSession destroy
    return $HOST_IS_UP
}


#
# check_host_p:
#
# this procedure is used exclusively by bringup to check
# signatures on this processor only.
# when reading MIB failed, check_host_p returns DOWN
#
proc check_host_p { SnmpSession cpu } {
    global HOST_IS_UP
    global HOST_IS_DOWN
    global CHECK_HOST_FAILED
    upvar bVerbose Verbose
    set sig ""
    set state ""

    if { $cpu < 0 || $cpu >= 64 } {
	if { $Verbose == 1 } {
	    set message [format "invalid processor number: %d" $cpu]
	    logDomainMessage $message "check_host"
	}
	$SnmpSession destroy
	return $CHECK_HOST_FAILED
    }

    set PROC_STATE_TBL \
	"procStateGenNum procStateGenPgmSignature procStateGenPgmState"
    catch {$SnmpSession walk x  "$PROC_STATE_TBL"  {
	set proc [lindex [lindex $x 0] 2]
	if { $proc == $cpu } {
	    # Found proc. Check signatures
	    set sig [lindex [lindex $x 1] 2]
	    set state [lindex [lindex $x 2] 2]
	    break
	}
    }   } error

    if { ($sig == "") || ($state == "") } {
	if { $Verbose == 1 } {
	    set message [format "signatures on proc %d are empty" $cpu ]
	    logDomainMessage $message "check_host"
	}
	set res $HOST_IS_DOWN
    } else {
	if { (($state == "run") &&
	(($sig == "download-helper") || ($sig == "obp") || ($sig == "os"))) } {
	    set res $HOST_IS_UP
	} else {
	    set res $HOST_IS_DOWN
	}
    }

    $SnmpSession destroy
    if { ($res == $HOST_IS_DOWN) && ($Verbose == 1) } {
	set message [format "proc %s: sig %s and state %s" $proc $sig $state]
	logDomainMessage $message "check_host"
    }
    return $res
}


#
# dump_signatures
# This proc is added for bringup -B, so it can tell us to log signatures
# into the domain messages file. NOTE: we break up signatures in small
# chunks, otherwise logger core dumps.
#
proc dump_signatures { SnmpSession index verbose } {
    set message {}
    set j 0

    if { $verbose == 0 } { return }

    logDomainMessage "INFO: Reporting CPU signatures:" "check_host"

    foreach i $index {
	if { [ catch {$SnmpSession get "procStateGenNum.$i \
		procStateGenPgmSignature.$i procStateGenPgmState.$i \
		procStateGenPgmSubState.$i"} x ] == 0 } {
	    append message [format "%d %s %s %s, " \
		    [lindex [lindex $x 0] 2] [lindex [lindex $x 1] 2] \
		    [lindex [lindex $x 2] 2] [lindex [lindex $x 3] 2] ]
	    incr j
	    if { $j == 10 } {
		logDomainMessage $message "check_host"
		#puts "check_host $message"
		set j 0
		set message {}
	    }
	}
    }
    logDomainMessage $message "check_host"
    #puts "check_host $message"
}


#
# get_platform_name
# Return the platform name for the given domain name.
# This is obtained from the domain_config file.
# (bug 4102350)
proc get_platform_name { domain_name } {
    foreach i [get_domain_config] {
	if { [lindex $i 0] == $domain_name } {
	    return [lindex $i 2]
	}
    }
}


#
# get_primary_cb_name
# Return the primary control board hostname for a given platform name.
# This is obtained from the cb_config file.
# (bug 4102350)
proc get_primary_cb_name { platform_name } {
    global CB_CONFIG_FILE

    if { [file exists $CB_CONFIG_FILE] == 0} {
        logMessage "Cannot access cb_config file."
        return
    }

    # The format we expect the cb_config file to be in is:
    #    platform_name:platform_type:cb0_name:cb0_status:cb1_name:cb1_status
    # (either cb0_status or cb1_status (but not both) is "P")
    # E.g.,
    # xf8:Ultra-Enterprise-10000:xf8-cb0:P:xf8-cb1:B

    set f [open $CB_CONFIG_FILE r]
    while {[gets $f rawline] >= 0} {
	set cb_entry [split $rawline :]
	if { [lindex $cb_entry 0] == $platform_name } {
	    # Entry found--now find out which CB is the primary
	    close $f
	    if {[lindex $cb_entry 3 ] == "P" } {
		return [lindex $cb_entry 2 ]
	    } elseif {[lindex $cb_entry 5 ] == "P" } {
		return [lindex $cb_entry 4 ]
	    } else {
		logMessage "No primary control board in cb_config for platform $platform_name"
		return
	    }
	}
    }
    close $f
}
